'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var shallowEqual = _interopDefault(require('shallowequal'));
var createReactContext = _interopDefault(require('create-react-context'));

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _await(value, then, direct) {
  if (direct) {
    return then ? then(value) : value;
  }

  if (!value || !value.then) {
    value = Promise.resolve(value);
  }

  return then ? value.then(then) : value;
}

var defaultErrorNormalizer = function defaultErrorNormalizer(e) {
  return (e == null ? void 0 : e.message) || 'Unknown error';
};

function _empty() {}

var defaultOptionsComparator = shallowEqual;

function _invokeIgnored(body) {
  var result = body();

  if (result && result.then) {
    return result.then(_empty);
  }
}

var isFunc = function isFunc(f) {
  return typeof f === 'function';
};

function _catch(body, recover) {
  try {
    var result = body();
  } catch (e) {
    return recover(e);
  }

  if (result && result.then) {
    return result.then(void 0, recover);
  }

  return result;
}

var isPromise = function isPromise(p) {
  return p && isFunc(p.then);
};

var EMPTY_DATA_STATE = {
  data: null,
  loading: true,
  error: null,
  rawError: null,
  loadedOnce: false,
  options: {}
};
/**
 * Provide a way to set default error normalization function
 * that will be used if no specific one provided in the loader
 * @param {Function} normalizer
 */

var setDefaultErrorNormalizer = function setDefaultErrorNormalizer(normalizer) {
  return defaultErrorNormalizer = normalizer;
};
/**
 * Provide a way to set default options comparator function
 * that will be used if no specific one provided in the loader
 * @param {Function} normalizer
 */

var setDefaultOptionsComparator = function setDefaultOptionsComparator(comparator) {
  return defaultOptionsComparator = comparator;
};
/**
 * A helper logic component that handle data loading on mount of the component.
 * It notify about state changes using render prop and provided handelr functions.
 */

function createDataLoader(createParts, createOptions) {
  var optionsComparator = (createOptions == null ? void 0 : createOptions.optionsComparator) || defaultOptionsComparator;
  var errorNormalizer = (createOptions == null ? void 0 : createOptions.errorNormalizer) || defaultErrorNormalizer;
  var DataContext = createReactContext({
    state: EMPTY_DATA_STATE,
    actions: {}
  });

  var DataLoader =
  /*#__PURE__*/
  function (_React$Component) {
    _inheritsLoose(DataLoader, _React$Component);

    function DataLoader(props) {
      var _this;

      _this = _React$Component.call(this, props) || this;

      _defineProperty(_assertThisInitialized(_this), "unmounted", false);

      _defineProperty(_assertThisInitialized(_this), "iteration", 0);

      _defineProperty(_assertThisInitialized(_this), "parts", void 0);

      _defineProperty(_assertThisInitialized(_this), "loadNextData", function () {
        _this.incrementIteration();

        _this.doLoadData(_this.iteration);
      });

      _this.incrementIteration();

      _this.state = _objectSpread2({}, EMPTY_DATA_STATE, {}, props.state, {
        options: props.options
      });
      return _this;
    }

    var _proto = DataLoader.prototype;

    _proto.componentWillUnmount = function componentWillUnmount() {
      this.unmounted = true;
      this.iteration = -1;
      this.cleanupActiveParts();
      var _this$props = this.props,
          onStateUpdate = _this$props.onStateUpdate,
          emptyOnUnmount = _this$props.emptyOnUnmount;
      var activeState = this.activeState;

      if (emptyOnUnmount && onStateUpdate) {
        // Empty the state usually needed when the user left the page
        // so state for the previos page is not needed anymore. To avoid
        // any kind of flickering of the previous page, empty the state in
        // a little delay
        setTimeout(function () {
          onStateUpdate(_objectSpread2({}, activeState, {}, EMPTY_DATA_STATE));
        }, 60);
      }
    };

    _proto.componentDidMount = function componentDidMount() {
      var activeState = this.activeState;

      if (!this.props.lazyReloadOnMount || !activeState.loadedOnce || activeState.loading || !optionsComparator(activeState.options, this.props.options)) {
        this.doLoadData(this.iteration);
      }
    };

    _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
      if (!optionsComparator(prevProps.options, this.props.options)) {
        this.loadNextData();
      }
    };

    _proto.cleanupActiveParts = function cleanupActiveParts() {
      if (this.parts && this.parts.cleanup) {
        this.parts.cleanup();
      }
    };

    _proto.incrementIteration = function incrementIteration() {
      var iteration = ++this.iteration;
      var self = this;
      var currentOptions = self.props.options;
      var immutableState = {
        setData: function setData(data) {
          if (iteration === self.iteration) {
            self.setStateWithNotify({
              data: data
            });
          }
        },
        retry: function retry() {
          if (iteration === self.iteration) {
            self.loadNextData();
          }
        },

        get iteration() {
          return iteration;
        },

        get destroyed() {
          return iteration !== self.iteration;
        },

        get data() {
          return self.activeState.data;
        },

        get options() {
          return currentOptions;
        },

        get loading() {
          return self.activeState.loading;
        },

        get error() {
          return self.activeState.error;
        },

        get rawError() {
          return self.activeState.rawError;
        },

        get loadedOnce() {
          return self.activeState.loadedOnce;
        }

      };
      this.cleanupActiveParts();
      this.parts = createParts(immutableState);
    };

    _proto.shouldLoadData = function shouldLoadData() {
      var shouldLoadData = this.props.shouldLoadData || (createOptions == null ? void 0 : createOptions.defaultShouldLoadData);
      return shouldLoadData ? !!shouldLoadData(this.props.options) : true;
    };

    _proto.doLoadData = function doLoadData(iteration) {
      try {
        var _this3 = this;

        return _catch(function () {
          if (!_this3.shouldLoadData()) return;

          var rawResult = _this3.parts.loader();

          return _invokeIgnored(function () {
            if (isPromise(rawResult)) {
              _this3.setStateWithNotify({
                options: _this3.props.options,
                loading: true,
                error: null,
                rawError: null
              });

              return _await(rawResult, function (resolvedResult) {
                if (iteration === _this3.iteration) {
                  _this3.handleDataLoaded(resolvedResult);
                }
              });
            } else {
              _this3.handleDataLoaded(rawResult);
            }
          });
        }, function (e) {
          if (iteration === _this3.iteration) {
            _this3.handleErrorWhileLoading(e);
          }
        });
      } catch (e) {
        return Promise.reject(e);
      }
    };

    _proto.setStateWithNotify = function setStateWithNotify(updateState) {
      var _this4 = this;

      if (this.unmounted) return;

      if (this.isStateControlled) {
        if (this.props.onStateUpdate) {
          this.props.onStateUpdate(_objectSpread2({}, this.activeState, {}, updateState));
        }
      } else {
        this.setState(updateState, function () {
          if (!_this4.unmounted && _this4.props.onStateUpdate) {
            _this4.props.onStateUpdate(_this4.activeState);
          }
        });
      }
    };

    _proto.handleErrorWhileLoading = function handleErrorWhileLoading(error) {
      if (this.unmounted) return;
      var normError = errorNormalizer(error);
      this.setStateWithNotify({
        options: this.props.options,
        loading: false,
        loadedOnce: true,
        error: normError,
        rawError: error
      });
    };

    _proto.handleDataLoaded = function handleDataLoaded(data) {
      if (this.unmounted) return;
      this.setStateWithNotify({
        options: this.props.options,
        loading: false,
        loadedOnce: true,
        error: null,
        rawError: null,
        data: data
      });
    };

    _proto.render = function render() {
      var actualActions = _objectSpread2({
        retry: this.loadNextData
      }, this.parts.actions);

      var contextValue = {
        state: this.activeState,
        actions: actualActions
      };
      var content = typeof this.props.children === 'function' ? this.props.children(contextValue) : this.props.children;
      return React.createElement(DataContext.Provider, {
        value: contextValue
      }, content);
    };

    _createClass(DataLoader, [{
      key: "isStateControlled",
      get: function get() {
        return this.props.state !== undefined && this.props.onStateUpdate;
      }
    }, {
      key: "activeState",
      get: function get() {
        return this.isStateControlled ? this.props.state || this.state : this.state;
      }
    }]);

    return DataLoader;
  }(React.Component);

  _defineProperty(DataLoader, "displayName", ((createOptions == null ? void 0 : createOptions.displayName) || 'Data') + ".Loader");

  DataContext.Loader = DataLoader;
  DataContext.Factory = createParts;
  return DataContext;
}

exports.createDataLoader = createDataLoader;
exports.default = createDataLoader;
exports.setDefaultErrorNormalizer = setDefaultErrorNormalizer;
exports.setDefaultOptionsComparator = setDefaultOptionsComparator;
